####### Code for bioage estimation###########
######Author: Xu Gao, Peking University######
############## 2023.2.20 ####################

#0. Clean the UK biobank data prior to use the code below by renaming corresponding variables as named in "BioAge" package
#install package "BioAge"
install.packages("devtools")
devtools::install_github("dayoonkwon/BioAge")

#1. Load packages
library(BioAge)
library(dplyr)
library(readr)
library(foreign)

#2. Read prepared data
UKB <- read_csv("D:/Data/UKB/bioage.csv")

#specify bioamarkers included in the algorithms 
biomarkers_kdm = c("fev","sbp","totchol","hba1c","albumin","creat","lncrp","alp","bun")

biomarkers_phenoage = c("albumin_gL","lymph","mcv","glucose_mmol","rdw","creat_umol","lncrp","alp","wbc")

#KDM bioage using NHANES (separate training for men and women)
kdm = kdm_nhanes(biomarkers_kdm)

#phenoage uinsg NHANES
phenoage = phenoage_nhanes(biomarkers_phenoage)

########################For UKB##############################
#kdm-age
kdm_female = kdm_calc(data = UKB %>%
                     filter (sex ==0),
                   biomarkers= biomarkers_kdm,
                   fit = kdm$fit$female,
                   s_ba2 = kdm$fit$female$s_b2)

kdm_male = kdm_calc(data = UKB %>%
                      filter (sex ==1),
                    biomarkers= biomarkers_kdm,
                    fit = kdm$fit$male,
                    s_ba2 = kdm$fit$male$s_b2)

kdm_ukb = rbind(kdm_female$data, kdm_male$data)

#PhenoAge
phenoage_ukb = phenoage_calc(data = UKB %>%
                               mutate(albumin = albumin_gL,
                                      lncreat = lncreat_umol),
                             biomarkers=biomarkers_phenoage,
                             fit = phenoage$fit,
                             orig = TRUE) 

phenoage_ukb = phenoage_ukb$data

#pull the full dataset
UKB =left_join(UKB, kdm_ukb[, c("sampleID", "kdm", "kdm_advance")], by = "sampleID") %>%
  left_join(., phenoage_ukb[, c("sampleID", "phenoage","phenoage_advance")], by = "sampleID") 

#export data for SAS analysis
write.foreign(df=UKB, datafile="D:/test.csv", codefile="D:/test.sas", package="SAS")
